@@ -1,13 +1,15 @@
-
-__version__ = "0.0.2"
-
-
 import base64
-from collections import defaultdict
+import datetime
+import decimal
 import hashlib
 import hmac
-import urllib
-import urlparse
+import json
+import six
+
+if six.PY3:
+    from urllib.parse import urlparse
+else:
+    from urlparse import urlparse
 
 
 __all__ = (
@@ -15,12 +17,6 @@ __all__ = (
     'get_signature',
 )
 
-def is_list(v):
-    return isinstance(v, (list, tuple))
-
-def sort_vals(vals):
-    return sorted(vals) if is_list(vals) else vals
-
 
 def get_signature(private_key, base_url, payload=None):
     """
@@ -38,6 +34,19 @@ def get_signature(private_key, base_url, payload=None):
     return Signer(private_key).create_signature(base_url, payload)
 
 
+class DefaultJSONEncoder(json.JSONEncoder):
+    """
+    JSONEncoder subclass that knows how to encode date/time and decimal types.
+    """
+    def default(self, o):
+        if isinstance(o, (datetime.datetime, datetime.date, datetime.time)):
+            return o.isoformat()
+        elif isinstance(o, decimal.Decimal):
+            return str(o)
+        else:
+            return super(DefaultJSONEncoder, self).default(o)
+
+
 class Signer(object):
     """
     Creates an HMAC signature for a url and possible POST data payload.
@@ -65,34 +74,20 @@ class Signer(object):
         :param payload:
             The POST data that you'll be sending.
         """
-        url = urlparse.urlparse(base_url)
+        url = urlparse(base_url)
 
-        url_to_sign = url.path + '?' + url.query
-        encoded_payload = self._encode_payload(payload)
+        url_to_sign = "{path}?{query}".format(path=url.path, query=url.query)
+
+        converted_payload = self._convert(payload)
 
         decoded_key = base64.urlsafe_b64decode(self.private_key.encode('utf-8'))
-        signature = hmac.new(decoded_key, url_to_sign + encoded_payload, hashlib.sha256)
-        return base64.urlsafe_b64encode(signature.digest())
+        signature = hmac.new(decoded_key, str.encode(url_to_sign + converted_payload), hashlib.sha256)
+        return bytes.decode(base64.urlsafe_b64encode(signature.digest()))
 
-    def _encode_payload(self, payload):
+    def _convert(self, payload):
         """
-        Ensures the order of items coming from urlencode are the same
-        every time so we can reliably recreate the signature.
-
-        :param payload:
-            The data to be sent in a POST request.
-            Can be a dictionary or it can be an iterable of
-            two items, first being key, second being value(s).
+        Converts payload to a string. Complex objects are dumped to json
         """
-        if payload is None:
-            return ''
-
-        if hasattr(payload, 'items'):
-            payload = payload.items()
-
-        p = defaultdict(list)
-        for k, v in payload:
-            p[k].extend(v) if is_list(v) else p[k].append(v)
-        ordered_params = [(k, sort_vals(p[k])) for k in sorted(p.keys())]
-
-        return urllib.urlencode(ordered_params, True)
+        if not isinstance(payload, six.string_types) and payload:
+            payload = json.dumps(payload, cls=DefaultJSONEncoder, sort_keys=True)
+        return str(payload or "")
